# Copyright 2017 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections

from sushy.resources import base

from proliantutils.redfish.resources.system import constants as sys_cons
from proliantutils.redfish.resources.system import mappings

MemoryData = collections.namedtuple(
    'MemoryData', ['has_persistent_memory',
                   'has_nvdimm_n',
                   'has_logical_nvdimm_n'])


class Memory(base.ResourceBase):

    identity = base.Field('Id')
    """The memory identity string"""

    name = base.Field('Name')
    """The name of the resource or array element"""

    description = base.Field('Description')
    """Description"""

    memory_device_type = base.MappedField('MemoryDeviceType',
                                          mappings.MEMORY_DEVICE_TYPE_MAP)
    memory_type = base.MappedField('MemoryType',
                                   mappings.MEMORY_TYPE_MAP)


class MemoryCollection(base.ResourceCollectionBase):

    @property
    def _resource_type(self):
        return Memory

    def details(self):
        persistent_memory = False
        nvdimm_n = False
        logical_nvdimm_n = False
        for mem in self.get_members():
            if mem.memory_type == sys_cons.MEMORY_TYPE_NVDIMM_N:
                persistent_memory = True
                nvdimm_n = True
                if (mem.memory_device_type
                        == sys_cons.MEMORY_DEVICE_TYPE_LOGICAL):
                    logical_nvdimm_n = True
                    break

        return MemoryData(has_persistent_memory=persistent_memory,
                          has_nvdimm_n=nvdimm_n,
                          has_logical_nvdimm_n=logical_nvdimm_n)



# Copyright 2017 Hewlett Packard Enterprise Development LP
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sushy.resources import base
from sushy import utils as sushy_utils

from proliantutils.redfish.resources.system.storage import mappings
from proliantutils.redfish import utils


class HPELogicalDrive(base.ResourceBase):
    """This class represents the LogicalDrives resource"""

    identity = base.Field('Id')

    name = base.Field('Name')

    description = base.Field('Description')

    capacity_mib = base.Field('CapacityMiB', adapter=int)

    raid = base.MappedField('Raid', mappings.RAID_LEVEL_MAP)


class HPELogicalDriveCollection(base.ResourceCollectionBase):
    """This class represents the collection of LogicalDrives resource"""

    @property
    def _resource_type(self):
        return HPELogicalDrive

    @property
    @sushy_utils.cache_it
    def maximum_size_mib(self):
        """Gets the biggest logical drive

        :returns size in MiB.
        """
        return utils.max_safe([member.capacity_mib
                               for member in self.get_members()])

    @property
    @sushy_utils.cache_it
    def logical_raid_levels(self):
        """Gets the raid level for each logical volume

        :returns the set of list of raid levels configured.
        """
        lg_raid_lvls = set()
        for member in self.get_members():
            lg_raid_lvls.add(mappings.RAID_LEVEL_MAP_REV.get(member.raid))
        return lg_raid_lvls



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

INTERFACE_TYPE_SAS = 'sas'
INTERFACE_TYPE_SCSI = 'scsi'
INTERFACE_TYPE_SATA = 'sata'

DISK_TYPE_HDD = 'hdd'
DISK_TYPE_SSD = 'ssd'

RAID_0 = '0'
RAID_1 = '1'
RAID_10 = '1+0'
RAID_5 = '5'
RAID_6 = '6'
RAID_50 = '5+0'
RAID_60 = '6+0'
RAID_1_0 = '10'
RAID_5_0 = '50'
RAID_6_0 = '60'
# Below are not supported in Ironic now.
RAID_1_ADM = '1ADM'
RAID_10_ADM = '10ADM'

RAID_LEVEL_INPUT_TO_HPSSA_MAPPING = {RAID_50: '50', RAID_60: '60'}
RAID_LEVEL_HPSSA_TO_INPUT_MAPPING = {
    v: k for k, v in RAID_LEVEL_INPUT_TO_HPSSA_MAPPING.items()}

INTERFACE_TYPE_MAP = {'SCSI': INTERFACE_TYPE_SCSI,
                      'SAS': INTERFACE_TYPE_SAS,
                      'SATA': INTERFACE_TYPE_SATA,
                      'SATASSD': INTERFACE_TYPE_SATA,
                      'SASSSD': INTERFACE_TYPE_SAS,
                      'Solid State SAS': INTERFACE_TYPE_SAS,
                      'Solid State SATA': INTERFACE_TYPE_SATA}

DISK_TYPE_MAP = {'SCSI': DISK_TYPE_HDD,
                 'SAS': DISK_TYPE_HDD,
                 'SATA': DISK_TYPE_HDD,
                 'SATASSD': DISK_TYPE_SSD,
                 'SASSSD': DISK_TYPE_SSD,
                 'Solid State SAS': DISK_TYPE_SSD,
                 'Solid State SATA': DISK_TYPE_SSD}

RAID_LEVEL_MIN_DISKS = {RAID_0: 1,
                        RAID_1: 2,
                        RAID_1_ADM: 3,
                        RAID_5: 3,
                        RAID_6: 4,
                        RAID_10: 4,
                        RAID_1_0: 4,
                        RAID_50: 6,
                        RAID_5_0: 6,
                        RAID_60: 8,
                        RAID_6_0: 8}


MINIMUM_DISK_SIZE = 1


def get_interface_type(ssa_interface):
    return INTERFACE_TYPE_MAP[ssa_interface]


def get_disk_type(ssa_interface):
    return DISK_TYPE_MAP[ssa_interface]



# Copyright 2022 Hewlett Packard Enterprise Development LP
# Copyright 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""Test class for Client Module."""
from unittest import mock

import testtools

from proliantutils import exception
from proliantutils.ilo import client
from proliantutils.ilo import ipmi
from proliantutils.ilo import ribcl
from proliantutils.ilo import ris
from proliantutils.ilo.snmp import snmp_cpqdisk_sizes
from proliantutils.redfish import redfish


def get_cls_wrapper(cls, cache=True):
    original_cls = cls
    cls = client.cache_node(cache)(cls)
    return (original_cls, cls)


class IloCacheNodeTestCase(testtools.TestCase):

    def test_cache_node_cache_true(self):
        class Dummyclass1(object):
            pass

        original_cls, decorated_cls = get_cls_wrapper(Dummyclass1)
        self.assertNotEqual(id(original_cls), id(decorated_cls))

    def test_cache_node_cache_false(self):
        class Dummyclass2(object):
            pass

        original_cls, decorated_cls = get_cls_wrapper(Dummyclass2,
                                                      cache=False)
        self.assertEqual(id(original_cls), id(decorated_cls))


class IloClientWrapperTestCase(testtools.TestCase):
    class DummyClass(object):
        def __init__(self, ip, name, password):
            self._ip = ip
            self._name = name
            self._password = password

    original_cls, decorated_cls = get_cls_wrapper(DummyClass)

    wrapper_cls = decorated_cls.__class__

    @mock.patch.object(wrapper_cls, '_create_instance')
    @mock.patch.object(wrapper_cls, '_if_not_exists')
    def test___call___already_created(self, exists_mock, create_mock):
        exists_mock.return_value = True
        try:
            wrapper_obj = IloClientWrapperTestCase.wrapper_cls(
                IloClientWrapperTestCase.original_cls)
            wrapper_obj('a.b.c.d', 'abcd', 'deaf')
        except KeyError:
            pass
        exists_mock.assert_called_once_with(('a.b.c.d', 'abcd', 'deaf'))
        create_mock.assert_called_once_with('a.b.c.d', 'abcd', 'deaf')

    @mock.patch.object(wrapper_cls, '_create_instance')
    @mock.patch.object(wrapper_cls, '_if_not_exists')
    def test___call___new(self, exists_mock, create_mock):
        exists_mock.return_value = False
        try:
            wrapper_obj = IloClientWrapperTestCase.wrapper_cls(
                IloClientWrapperTestCase.original_cls)
            wrapper_obj('a.b.c.d', 'abcd', 'deaf')
        except KeyError:
            pass
        exists_mock.assert_called_once_with(('a.b.c.d', 'abcd', 'deaf'))
        create_mock.assert_not_called()

    @mock.patch.object(original_cls, '__init__')
    @mock.patch.object(wrapper_cls, '_pop_oldest_node')
    def test__create_instance(self, pop_mock, init_mock):
        init_mock.return_value = None
        wrapper_obj = IloClientWrapperTestCase.wrapper_cls(
            IloClientWrapperTestCase.original_cls)
        wrapper_obj.MAX_CACHE_SIZE = 2
        wrapper_obj._create_instance('a.b.c.d', 'abcd', 'defe')
        init_mock.assert_called_once_with('a.b.c.d', 'abcd', 'defe')
        pop_mock.assert_not_called()

    @mock.patch.object(original_cls, '__init__')
    @mock.patch.object(wrapper_cls, '_pop_oldest_node')
    def test__create_instance_max_size(self, pop_mock, init_mock):
        init_mock.return_value = None
        wrapper_obj = IloClientWrapperTestCase.wrapper_cls(
            IloClientWrapperTestCase.original_cls)
        wrapper_obj.MAX_CACHE_SIZE = 2
        wrapper_obj._create_instance('a.b.c.d', 'abcd', 'deaf')
        wrapper_obj._create_instance('e.f.g.h', 'efgh', 'deaf')
        wrapper_obj._create_instance('i.j.k.l', 'ijkl', 'deaf')
        pop_mock.assert_called_once_with()

    def test__pop_oldest_node(self):
        wrapper_obj = IloClientWrapperTestCase.wrapper_cls(
            IloClientWrapperTestCase.original_cls)
        wrapper_obj.MAX_CACHE_SIZE = 2
        wrapper_obj('a.b.c.d', 'abcd', 'deaf')
        wrapper_obj('e.f.g.h', 'efgh', 'deaf')
        wrapper_obj('i.j.k.l', 'ijkl', 'deaf')
        self.assertIn(('i.j.k.l', 'ijkl', 'deaf'), wrapper_obj._instances)
        self.assertIn(('e.f.g.h', 'efgh', 'deaf'), wrapper_obj._instances)
        self.assertNotIn(('a.b.c.d', 'ijkl', 'deaf'), wrapper_obj._instances)


class IloClientInitTestCase(testtools.TestCase):

    @mock.patch.object(ribcl, 'RIBCLOperations')
    @mock.patch.object(ris, 'RISOperations')
    def test_init(self, ris_mock, ribcl_mock):
        ribcl_obj_mock = mock.MagicMock()
        ribcl_mock.return_value = ribcl_obj_mock
        ribcl_obj_mock.get_product_name.return_value = 'product'

        c = client.IloClient.cls("1.2.3.4", "admin", "Admin",
                                 timeout=120, port=4430,
                                 bios_password='foo',
                                 cacert='/somewhere')

        ris_mock.assert_called_once_with(
            "1.2.3.4", "admin", "Admin", bios_password='foo',
            cacert='/somewhere')
        ribcl_mock.assert_called_once_with(
            "1.2.3.4", "admin", "Admin", 120, 4430, cacert='/somewhere')
        self.assertEqual(
            {'address': "1.2.3.4", 'username': "admin", 'password': "Admin"},
            c.ipmi_host_info)
        self.assertEqual('product', c.model)

    @mock.patch.object(ribcl, 'RIBCLOperations')
    @mock.patch.object(ris, 'RISOperations')
    def test_init_for_ipv6_link_address(self, ris_mock, ribcl_mock):
        ribcl_obj_mock = mock.MagicMock()
        ribcl_mock.return_value = ribcl_obj_mock
        ribcl_obj_mock.get_product_name.return_value = 'product'

        c = client.IloClient.cls("FE80::9AF2:B3FF:FEEE:F884%eth0", "admin",
                                 "Admin", timeout=120, port=4430,
                                 bios_password='foo',
                                 cacert='/somewhere')

        ris_mock.assert_called_once_with(
            "[FE80::9AF2:B3FF:FEEE:F884%eth0]",
            "admin", "Admin", bios_password='foo',
            cacert='/somewhere')
        ribcl_mock.assert_called_once_with(
            "[FE80::9AF2:B3FF:FEEE:F884%eth0]",
            "admin", "Admin", 120, 4430, cacert='/somewhere')
        self.assertEqual(
            {'address': "FE80::9AF2:B3FF:FEEE:F884%eth0",
             'username': "admin", 'password': "Admin"},
            c.ipmi_host_info)
        self.assertEqual('product', c.model)

    @mock.patch.object(ribcl, 'RIBCLOperations')
    @mock.patch.object(ris, 'RISOperations')
    def test_init_for_ipv6_global_address(self, ris_mock, ribcl_mock):
        ribcl_obj_mock = mock.MagicMock()
        ribcl_mock.return_value = ribcl_obj_mock
        ribcl_obj_mock.get_product_name.return_value = 'product'

        c = client.IloClient.cls("2001:0db8:85a3::8a2e:0370:7334", "admin",
                                 "Admin", timeout=120, port=4430,
                                 bios_password='foo',
                                 cacert='/somewhere')

        ris_mock.assert_called_once_with(
            "[2001:0db8:85a3::8a2e:0370:7334]",
            "admin", "Admin", bios_password='foo',
            cacert='/somewhere')
        ribcl_mock.assert_called_once_with(
            "[2001:0db8:85a3::8a2e:0370:7334]",
            "admin", "Admin", 120, 4430, cacert='/somewhere')
        self.assertEqual(
            {'address': "2001:0db8:85a3::8a2e:0370:7334",
             'username': "admin", 'password': "Admin"},
            c.ipmi_host_info)
        self.assertEqual('product', c.model)

    @mock.patch.object(ribcl, 'RIBCLOperations')
    @mock.patch.object(redfish, 'RedfishOperations')
    def test_init_for_redfish_with_ribcl_enabled(
            self, redfish_mock, ribcl_mock):
        ribcl_obj_mock = mock.MagicMock()
        ribcl_mock.return_value = ribcl_obj_mock
        ribcl_obj_mock.get_product_name.return_value = 'ProLiant DL180 Gen10'

        c = client.IloClient.cls("1.2.3.4", "admin", "Admin",
                                 timeout=120, port=4430,
                                 bios_password='foo',
                                 cacert='/somewhere')

        ribcl_mock.assert_called_once_with(
            "1.2.3.4", "admin", "Admin", 120, 4430, cacert='/somewhere')
        redfish_mock.assert_called_once_with(
            "1.2.3.4", "admin", "Admin", bios_password='foo',
            cacert='/somewhere')
        self.assertEqual(
            {'address': "1.2.3.4", 'username': "admin", 'password': "Admin"},
            c.ipmi_host_info)
        self.assertEqual('ProLiant DL180 Gen10', c.model)
        self.assertIsNotNone(c.redfish)
        self.assertTrue(c.is_ribcl_enabled)
        self.assertFalse(hasattr(c, 'ris'))

    @mock.patch.object(ribcl, 'RIBCLOperations')
    @mock.patch.object(redfish, 'RedfishOperations')
    def test_init_for_redfish_with_ribcl_disabled(
            self, redfish_mock, ribcl_mock):
        ribcl_obj_mock = mock.MagicMock()
        ribcl_mock.return_value = ribcl_obj_mock
        ribcl_obj_mock.get_product_name.side_effect = (
            exception.IloError('RIBCL is disabled'))

        c = client.IloClient.cls("1.2.3.4", "admin", "Admin",
                                 timeout=120, port=4430,
                                 bios_password='foo',
                                 cacert='/somewhere')

        ribcl_mock.assert_called_once_with(
            "1.2.3.4", "admin", "Admin", 120, 4430, cacert='/somewhere')
        redfish_mock.assert_called_once_with(
            "1.2.3.4", "admin", "Admin", bios_password='foo',
            cacert='/somewhere')
        self.assertEqual(
            {'address': "1.2.3.4", 'username': "admin", 'password': "Admin"},
            c.ipmi_host_info)
        self.assertIsNotNone(c.model)
        self.assertIsNotNone(c.redfish)
        self.assertFalse(c.is_ribcl_enabled)
        self.assertFalse(hasattr(c, 'ris'))

    @mock.patch.object(ribcl, 'RIBCLOperations')
    @mock.patch.object(redfish, 'RedfishOperations')
    def test_init_with_use_redfish_only_set(
            self, redfish_mock, ribcl_mock):
        c = client.IloClient.cls("1.2.3.4", "admin", "Admin",
                                 timeout=120, port=4430,
                                 bios_password='foo', cacert='/somewhere',
                                 use_redfish_only=True)
        ribcl_mock.assert_called_once_with(
            "1.2.3.4", "admin", "Admin", 120, 4430, cacert='/somewhere')
        redfish_mock.assert_called_once_with(
            "1.2.3.4", "admin", "Admin", bios_password='foo',
            cacert='/somewhere')
        self.assertEqual(
            {'address': "1.2.3.4", 'username': "admin", 'password': "Admin"},
            c.ipmi_host_info)
        self.assertIsNotNone(c.model)
        self.assertIsNotNone(c.redfish)
        self.assertIsNone(c.is_ribcl_enabled)
        self.assertFalse(hasattr(c, 'ris'))
        self.assertTrue(c.use_redfish_only)

    @mock.patch.object(client.IloClient.cls, '_validate_snmp')
    @mock.patch.object(ribcl, 'RIBCLOperations')
    @mock.patch.object(ris, 'RISOperations')
    def test_init_snmp(self, ris_mock, ribcl_mock, snmp_mock):
        ribcl_obj_mock = mock.MagicMock()
        ribcl_mock.return_value = ribcl_obj_mock
        ribcl_obj_mock.get_product_name.return_value = 'product'
        snmp_credentials = {'auth_user': 'user',
                            'auth_protocol': 'SHA',
                            'auth_prot_pp': '1234',
                            'priv_protocol': 'AES',
                            'auth_priv_pp': '4321',
                            'snmp_inspection': 'true'}

        c = client.IloClient.cls("1.2.3.4", "admin", "Admin",
                                 timeout=120, port=4430,
                                 bios_password='foo',
                                 cacert='/somewhere',
                                 snmp_credentials=snmp_credentials)

        ris_mock.assert_called_once_with(
            "1.2.3.4", "admin", "Admin", bios_password='foo',
            cacert='/somewhere')
        ribcl_mock.assert_called_once_with(
            "1.2.3.4", "admin", "Admin", 120, 4430, cacert='/somewhere')
        self.assertEqual(
            {'address': "1.2.3.4", 'username': "admin", 'password': "Admin"},
            c.ipmi_host_info)
        self.assertEqual('product', c.model)
        self.assertTrue(snmp_mock.called)

    @mock.patch.object(client.IloClient.cls, '_validate_snmp')
    @mock.patch.object(ribcl, 'RIBCLOperations')
    @mock.patch.object(ris, 'RISOperations')
    def test_init_snmp_raises(self, ris_mock, ribcl_mock, snmp_mock):
        ribcl_obj_mock = mock.MagicMock()
        ribcl_mock.return_value = ribcl_obj_mock
        ribcl_obj_mock.get_product_name.return_value = 'product'
        snmp_mock.side_effect = exception.IloInvalidInputError("msg")
        snmp_credentials = {'auth_user': 'user',
                            'auth_protocol': 'SHA',
                            'priv_protocol': 'AES',
                            'snmp_inspection': 'true'}

        self.assertRaises(exception.IloInvalidInputError, client.IloClient.cls,
                          "1.2.3.4", "admin", "Admin",
                          timeout=120, port=4430,
                          bios_password='foo',
                          cacert='/somewhere',
                          snmp_credentials=snmp_credentials)

        ris_mock.assert_called_once_with(
            "1.2.3.4", "admin", "Admin", bios_password='foo',
            cacert='/somewhere')
        ribcl_mock.assert_called_once_with(
            "1.2.3.4", "admin", "Admin", 120, 4430, cacert='/somewhere')
        self.assertTrue(snmp_mock.called)


class IloClientSNMPValidateTestCase(testtools.TestCase):

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def test_validate_snmp(self, product_mock):
        cred = {'auth_user': 'user',
                'auth_protocol': 'SHA',
                'priv_protocol': 'AES',
                'auth_prot_pp': '1234',
                'auth_priv_pp': '4321',
                'snmp_inspection': True}
        self.snmp_credentials = cred
        self.client = client.IloClient.cls("1.2.3.4", "admin", "Admin",
                                           snmp_credentials=cred)
        self.assertEqual(self.client.snmp_credentials, cred)

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def test_validate_snmp_fail_auth_priv_pp_missing(self, product_mock):
        cred = {'auth_user': 'user',
                'auth_protocol': 'SHA',
                'priv_protocol': 'AES',
                'auth_prot_pp': '1234',
                'snmp_inspection': True}
        self.assertRaises(exception.IloInvalidInputError,
                          client.IloClient.cls,
                          "1.2.3.4", "admin", "Admin",
                          snmp_credentials=cred)

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def test_validate_snmp_auth_prot_pp_missing(self, product_mock):
        cred = {'auth_user': 'user',
                'auth_protocol': 'SHA',
                'priv_protocol': 'AES',
                'auth_priv_pp': '4321',
                'snmp_inspection': True}
        self.assertRaises(exception.IloInvalidInputError,
                          client.IloClient.cls,
                          "1.2.3.4", "admin", "Admin",
                          snmp_credentials=cred)

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def test_validate_snmp_auth_prot_priv_prot_missing(self, product_mock):
        cred = {'auth_user': 'user',
                'auth_prot_pp': '1234',
                'auth_priv_pp': '4321',
                'snmp_inspection': True}
        self.client = client.IloClient.cls("1.2.3.4", "admin", "Admin",
                                           snmp_credentials=cred)
        self.assertEqual(self.client.snmp_credentials, cred)

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def test_validate_snmp_auth_user_missing(self, product_mock):
        cred = {'auth_protocol': 'SHA',
                'priv_protocol': 'AES',
                'auth_priv_pp': '4321',
                'auth_prot_pp': '1234',
                'snmp_inspection': True}
        self.assertRaises(exception.IloInvalidInputError,
                          client.IloClient.cls,
                          "1.2.3.4", "admin", "Admin",
                          snmp_credentials=cred)


class IloClientTestCase(testtools.TestCase):

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def setUp(self, product_mock):
        super(IloClientTestCase, self).setUp()
        product_mock.return_value = 'Gen8'
        self.client = client.IloClient.cls("1.2.3.4", "admin", "Admin")

    @mock.patch.object(ribcl.RIBCLOperations, 'get_all_licenses')
    def test__call_method_ribcl(self, license_mock):
        self.client._call_method('get_all_licenses')
        license_mock.assert_called_once_with()

    @mock.patch.object(ris.RISOperations, 'get_host_power_status')
    def test__call_method_ris(self, power_mock):
        self.client.model = 'Gen9'
        self.client._call_method('get_host_power_status')
        power_mock.assert_called_once_with()

    @mock.patch.object(ribcl.RIBCLOperations, 'reset_ilo')
    def test__call_method_gen9_ribcl(self, ilo_mock):
        self.client.model = 'Gen9'
        self.client._call_method('reset_ilo')
        ilo_mock.assert_called_once_with()

    """
    Testing ``_call_method`` with Redfish support.

    Testing the redfish methods based on the following scenarios,
    which are depicted in this table::

      redfish  |  ribcl   | method implemented | name of test method
    supported? | enabled? |    on redfish?     |
    ===========|==========|====================|=============================
       true    |   true   |      true          | test__call_method_redfish_1
       true    |   true   |      false         | test__call_method_redfish_2
       true    |   false  |      true          | test__call_method_redfish_3
       true    |   false  |      false         | test__call_method_redfish_4
    ===========|==========|====================|=============================
    """

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    @mock.patch.object(redfish, 'RedfishOperations')
    def test__call_method_redfish_1(self, redfish_mock,
                                    ribcl_product_name_mock):
        ribcl_product_name_mock.return_value = 'Gen10'
        self.client = client.IloClient.cls("1.2.3.4", "admin", "secret")
        redfish_get_host_power_mock = (redfish.RedfishOperations.return_value.
                                       get_host_power_status)

        self.client._call_method('get_host_power_status')
        redfish_get_host_power_mock.assert_called_once_with()

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    @mock.patch.object(redfish, 'RedfishOperations')
    @mock.patch.object(ribcl.RIBCLOperations, 'reset_ilo')
    def test__call_method_redfish_2(self, ribcl_reset_ilo_mock,
                                    redfish_mock, ribcl_product_name_mock):
        ribcl_product_name_mock.return_value = 'Gen10'
        self.client = client.IloClient.cls("1.2.3.4", "admin", "secret")

        self.client._call_method('reset_ilo')
        ribcl_reset_ilo_mock.assert_called_once_with()

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    @mock.patch.object(redfish, 'RedfishOperations')
    def test__call_method_redfish_3(self, redfish_mock,
                                    ribcl_product_name_mock):
        ribcl_product_name_mock.side_effect = (
            exception.IloError('RIBCL is disabled'))
        redfish_mock.return_value.get_product_name.return_value = 'Gen10'
        self.client = client.IloClient.cls("1.2.3.4", "admin", "secret")
        redfish_get_host_power_mock = (redfish.RedfishOperations.return_value.
                                       get_host_power_status)

        self.client._call_method('get_host_power_status')
        redfish_get_host_power_mock.assert_called_once_with()

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    @mock.patch.object(redfish, 'RedfishOperations')
    def test__call_method_redfish_4(self, redfish_mock,
                                    ribcl_product_name_mock):
        ribcl_product_name_mock.side_effect = (
            exception.IloError('RIBCL is disabled'))
        redfish_mock.return_value.get_product_name.return_value = 'Gen10'
        self.client = client.IloClient.cls("1.2.3.4", "admin", "secret")

        self.assertRaises(NotImplementedError,
                          self.client._call_method, 'reset_ilo')

    @mock.patch.object(redfish, 'RedfishOperations',
                       spec_set=True, autospec=True)
    def test__call_method_with_use_redfish_only_set(self, redfish_mock):
        self.client = client.IloClient.cls("1.2.3.4", "admin", "secret",
                                           use_redfish_only=True)
        redfish_get_host_power_mock = (
            redfish.RedfishOperations.return_value.get_host_power_status)

        self.client._call_method('get_host_power_status')
        redfish_get_host_power_mock.assert_called_once_with()

    @mock.patch.object(redfish, 'RedfishOperations',
                       spec_set=True, autospec=True)
    def test__call_method_use_redfish_only_set_but_not_implemented(
            self, redfish_mock):
        self.client = client.IloClient.cls("1.2.3.4", "admin", "secret",
                                           use_redfish_only=True)

        self.assertRaises(NotImplementedError,
                          self.client._call_method, 'reset_ilo')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_set_http_boot_url(self, call_mock):
        self.client.set_http_boot_url('fake-url')
        call_mock.assert_called_once_with('set_http_boot_url', 'fake-url')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_set_iscsi_info(self, call_mock):
        self.client.set_iscsi_info('iqn.2011-07.com:example:123',
                                   '1', '10.10.1.23', '3260', 'CHAP',
                                   'user', 'password')
        call_mock.assert_called_once_with('set_iscsi_info',
                                          'iqn.2011-07.com:example:123',
                                          '1', '10.10.1.23', '3260',
                                          'CHAP', 'user', 'password', [])

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_iscsi_initiator_info(self, call_mock):
        self.client.get_iscsi_initiator_info()
        call_mock.assert_called_once_with('get_iscsi_initiator_info')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_unset_iscsi_info(self, call_mock):
        self.client.unset_iscsi_info()
        call_mock.assert_called_once_with('unset_iscsi_info', [])

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_set_iscsi_initiator_info(self, call_mock):
        self.client.set_iscsi_initiator_info('iqn.2011-07.com:example:123')
        call_mock.assert_called_once_with('set_iscsi_initiator_info',
                                          'iqn.2011-07.com:example:123')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_product_name(self, call_mock):
        self.client.get_product_name()
        call_mock.assert_called_once_with('get_product_name')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_all_licenses(self, call_mock):
        self.client.get_all_licenses()
        call_mock.assert_called_once_with('get_all_licenses')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_host_power_status(self, call_mock):
        self.client.get_host_power_status()
        call_mock.assert_called_once_with('get_host_power_status')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_http_boot_url(self, call_mock):
        self.client.get_http_boot_url()
        call_mock.assert_called_once_with('get_http_boot_url')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_one_time_boot(self, call_mock):
        self.client.get_one_time_boot()
        call_mock.assert_called_once_with('get_one_time_boot')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_vm_status(self, call_mock):
        self.client.get_vm_status('CDROM')
        call_mock.assert_called_once_with('get_vm_status', 'CDROM')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_press_pwr_btn(self, call_mock):
        self.client.press_pwr_btn()
        call_mock.assert_called_once_with('press_pwr_btn')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_reset_server(self, call_mock):
        self.client.reset_server()
        call_mock.assert_called_once_with('reset_server')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_hold_pwr_btn(self, call_mock):
        self.client.hold_pwr_btn()
        call_mock.assert_called_once_with('hold_pwr_btn')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_set_host_power(self, call_mock):
        self.client.set_host_power('ON')
        call_mock.assert_called_once_with('set_host_power', 'ON')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_set_one_time_boot(self, call_mock):
        self.client.set_one_time_boot('CDROM')
        call_mock.assert_called_once_with('set_one_time_boot', 'CDROM')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_insert_virtual_media(self, call_mock):
        self.client.insert_virtual_media(url='fake-url', device='FLOPPY')
        call_mock.assert_called_once_with('insert_virtual_media', 'fake-url',
                                          'FLOPPY')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_eject_virtual_media(self, call_mock):
        self.client.eject_virtual_media(device='FLOPPY')
        call_mock.assert_called_once_with('eject_virtual_media', 'FLOPPY')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_set_vm_status(self, call_mock):
        self.client.set_vm_status(device='FLOPPY', boot_option='BOOT_ONCE',
                                  write_protect='YES')
        call_mock.assert_called_once_with('set_vm_status', 'FLOPPY',
                                          'BOOT_ONCE', 'YES')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_current_boot_mode(self, call_mock):
        self.client.get_current_boot_mode()
        call_mock.assert_called_once_with('get_current_boot_mode')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_pending_boot_mode(self, call_mock):
        self.client.get_pending_boot_mode()
        call_mock.assert_called_once_with('get_pending_boot_mode')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_supported_boot_mode(self, call_mock):
        self.client.get_supported_boot_mode()
        call_mock.assert_called_once_with('get_supported_boot_mode')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_set_pending_boot_mode(self, call_mock):
        self.client.set_pending_boot_mode('UEFI')
        call_mock.assert_called_once_with('set_pending_boot_mode', 'UEFI')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_persistent_boot_device(self, call_mock):
        self.client.get_persistent_boot_device()
        call_mock.assert_called_once_with('get_persistent_boot_device')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_update_persistent_boot(self, call_mock):
        self.client.update_persistent_boot(['HDD'])
        call_mock.assert_called_once_with('update_persistent_boot', ['HDD'])

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_secure_boot_mode(self, call_mock):
        self.client.get_secure_boot_mode()
        call_mock.assert_called_once_with('get_secure_boot_mode')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_set_secure_boot_mode(self, call_mock):
        self.client.set_secure_boot_mode(True)
        call_mock.assert_called_once_with('set_secure_boot_mode', True)

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_reset_secure_boot_keys(self, call_mock):
        self.client.reset_secure_boot_keys()
        call_mock.assert_called_once_with('reset_secure_boot_keys')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_clear_secure_boot_keys(self, call_mock):
        self.client.clear_secure_boot_keys()
        call_mock.assert_called_once_with('clear_secure_boot_keys')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_reset_ilo_credential(self, call_mock):
        self.client.reset_ilo_credential('password')
        call_mock.assert_called_once_with('reset_ilo_credential', 'password')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_reset_ilo(self, call_mock):
        self.client.reset_ilo()
        call_mock.assert_called_once_with('reset_ilo')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_reset_bios_to_default(self, call_mock):
        self.client.reset_bios_to_default()
        call_mock.assert_called_once_with('reset_bios_to_default')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_host_uuid(self, call_mock):
        self.client.get_host_uuid()
        call_mock.assert_called_once_with('get_host_uuid')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_host_health_data(self, call_mock):
        self.client.get_host_health_data('fake-data')
        call_mock.assert_called_once_with('get_host_health_data', 'fake-data')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_host_health_present_power_reading(self, call_mock):
        self.client.get_host_health_present_power_reading('fake-data')
        call_mock.assert_called_once_with(
            'get_host_health_present_power_reading', 'fake-data')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_host_health_power_supplies(self, call_mock):
        self.client.get_host_health_power_supplies('fake-data')
        call_mock.assert_called_once_with('get_host_health_power_supplies',
                                          'fake-data')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_host_health_fan_sensors(self, call_mock):
        self.client.get_host_health_fan_sensors('fake-data')
        call_mock.assert_called_once_with('get_host_health_fan_sensors',
                                          'fake-data')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_host_health_temperature_sensors(self, call_mock):
        self.client.get_host_health_temperature_sensors('fake-data')
        call_mock.assert_called_once_with(
            'get_host_health_temperature_sensors', 'fake-data')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_host_health_at_a_glance(self, call_mock):
        self.client.get_host_health_at_a_glance('fake-data')
        call_mock.assert_called_once_with('get_host_health_at_a_glance',
                                          'fake-data')

    @mock.patch.object(ipmi, 'get_nic_capacity')
    @mock.patch.object(ribcl.RIBCLOperations,
                       'get_ilo_firmware_version_as_major_minor')
    @mock.patch.object(ribcl.RIBCLOperations, 'get_server_capabilities')
    def test_get_server_capabilities(self, cap_mock, maj_min_mock, nic_mock):
        info = {'address': "1.2.3.4", 'username': "admin", 'password': "Admin"}
        str_val = maj_min_mock.return_value = "2.10"
        nic_mock.return_value = '10Gb'
        cap_mock.return_value = {'ilo_firmware_version': '2.10',
                                 'rom_firmware_version': 'x',
                                 'server_model': 'Gen8',
                                 'pci_gpu_devices': '2'}
        capabilities = self.client.get_server_capabilities()
        expected_capabilities = {'ilo_firmware_version': '2.10',
                                 'rom_firmware_version': 'x',
                                 'server_model': 'Gen8',
                                 'pci_gpu_devices': '2',
                                 'nic_capacity': '10Gb'}
        cap_mock.assert_called_once_with()
        nic_mock.assert_called_once_with(self.client.ipmi_host_info, str_val)
        self.assertEqual(expected_capabilities, capabilities)
        self.assertEqual(info, self.client.ipmi_host_info)

    @mock.patch.object(ipmi, 'get_nic_capacity')
    @mock.patch.object(ribcl.RIBCLOperations,
                       'get_ilo_firmware_version_as_major_minor')
    @mock.patch.object(ribcl.RIBCLOperations, 'get_server_capabilities')
    def test_get_server_capabilities_no_nic(self, cap_mock, maj_min_mock,
                                            nic_mock):
        info = {'address': "1.2.3.4", 'username': "admin", 'password': "Admin"}
        str_val = maj_min_mock.return_value = '2.10'
        nic_mock.return_value = None
        cap_mock.return_value = {'ilo_firmware_version': '2.10',
                                 'rom_firmware_version': 'x',
                                 'server_model': 'Gen8',
                                 'pci_gpu_devices': '2'}
        capabilities = self.client.get_server_capabilities()
        expected_capabilities = {'ilo_firmware_version': '2.10',
                                 'rom_firmware_version': 'x',
                                 'server_model': 'Gen8',
                                 'pci_gpu_devices': '2'}
        cap_mock.assert_called_once_with()
        nic_mock.assert_called_once_with(self.client.ipmi_host_info, str_val)
        self.assertEqual(expected_capabilities, capabilities)
        self.assertEqual(info, self.client.ipmi_host_info)

    @mock.patch.object(ipmi, 'get_nic_capacity')
    @mock.patch.object(ribcl.RIBCLOperations,
                       'get_ilo_firmware_version_as_major_minor')
    @mock.patch.object(ribcl.RIBCLOperations, 'get_server_capabilities')
    def test_get_server_capabilities_no_firmware(self, cap_mock, maj_min_mock,
                                                 nic_mock):
        maj_min_mock.return_value = None
        nic_mock.return_value = None
        cap_mock.return_value = {'rom_firmware_version': 'x',
                                 'server_model': 'Gen8',
                                 'pci_gpu_devices': '2'}
        expected_capabilities = {'rom_firmware_version': 'x',
                                 'server_model': 'Gen8',
                                 'pci_gpu_devices': '2'}
        capabilities = self.client.get_server_capabilities()
        self.assertEqual(expected_capabilities, capabilities)
        nic_mock.assert_called_once_with(self.client.ipmi_host_info, None)

    @mock.patch.object(ipmi, 'get_nic_capacity')
    @mock.patch.object(ribcl.RIBCLOperations,
                       'get_ilo_firmware_version_as_major_minor')
    @mock.patch.object(ribcl.RIBCLOperations, 'get_server_capabilities')
    def test_get_server_capabilities_no_boot_modes(
            self, cap_mock, maj_min_mock, nic_mock):
        maj_min_mock.return_value = None
        nic_mock.return_value = None
        cap_mock.return_value = {'rom_firmware_version': 'x',
                                 'server_model': 'Gen8',
                                 'pci_gpu_devices': '2'}
        expected_capabilities = {'rom_firmware_version': 'x',
                                 'server_model': 'Gen8',
                                 'pci_gpu_devices': '2'}
        capabilities = self.client.get_server_capabilities()
        self.assertEqual(expected_capabilities, capabilities)
        nic_mock.assert_called_once_with(self.client.ipmi_host_info, None)

    @mock.patch.object(ris.RISOperations,
                       'get_ilo_firmware_version_as_major_minor')
    @mock.patch.object(ipmi, 'get_nic_capacity')
    @mock.patch.object(ris.RISOperations, 'get_server_capabilities')
    def test_get_server_capabilities_no_nic_Gen9(self, cap_mock, nic_mock,
                                                 mm_mock):
        str_val = mm_mock.return_value = '2.10'
        self.client.model = 'Gen9'
        nic_mock.return_value = None

        cap_mock.return_value = {'ilo_firmware_version': '2.10',
                                 'rom_firmware_version': 'x',
                                 'server_model': 'Gen9',
                                 'pci_gpu_devices': 2,
                                 'secure_boot': 'true'}
        capabilities = self.client.get_server_capabilities()
        cap_mock.assert_called_once_with()
        nic_mock.assert_called_once_with(self.client.ipmi_host_info, str_val)
        expected_capabilities = {'ilo_firmware_version': '2.10',
                                 'rom_firmware_version': 'x',
                                 'server_model': 'Gen9',
                                 'pci_gpu_devices': 2,
                                 'secure_boot': 'true'}
        self.assertEqual(expected_capabilities, capabilities)

    @mock.patch.object(ris.RISOperations,
                       'get_ilo_firmware_version_as_major_minor')
    @mock.patch.object(ipmi, 'get_nic_capacity')
    @mock.patch.object(ris.RISOperations, 'get_server_capabilities')
    def test_get_server_capabilities_Gen9(self, cap_mock, nic_mock, mm_mock):
        str_val = mm_mock.return_value = '2.10'
        self.client.model = 'Gen9'
        nic_mock.return_value = '10Gb'
        cap_mock.return_value = {'ilo_firmware_version': '2.10',
                                 'rom_firmware_version': 'x',
                                 'server_model': 'Gen9',
                                 'pci_gpu_devices': 2,
                                 'secure_boot': 'true'}
        capabilities = self.client.get_server_capabilities()
        cap_mock.assert_called_once_with()
        nic_mock.assert_called_once_with(self.client.ipmi_host_info, str_val)
        expected_capabilities = {'ilo_firmware_version': '2.10',
                                 'rom_firmware_version': 'x',
                                 'server_model': 'Gen9',
                                 'pci_gpu_devices': 2,
                                 'secure_boot': 'true',
                                 'nic_capacity': '10Gb'}
        self.assertEqual(expected_capabilities, capabilities)

    @mock.patch.object(redfish, 'RedfishOperations')
    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    @mock.patch.object(ipmi, 'get_nic_capacity')
    def test_get_server_capabilities_Gen10(self, ipmi_mock,
                                           ribcl_product_name_mock,
                                           redfish_mock):
        ribcl_product_name_mock.return_value = 'Gen10'
        self.client = client.IloClient("1.2.3.4", "admin", "secret")
        cap_mock = (redfish_mock.return_value.get_server_capabilities)
        self.client.get_server_capabilities()
        self.assertFalse(ipmi_mock.called)
        self.assertTrue(cap_mock.called)

    @mock.patch.object(ris.RISOperations,
                       'get_ilo_firmware_version_as_major_minor')
    @mock.patch.object(ribcl.RIBCLOperations, 'get_host_health_data')
    @mock.patch.object(ris.RISOperations,
                       '_get_number_of_gpu_devices_connected')
    @mock.patch.object(ipmi, 'get_nic_capacity')
    @mock.patch.object(ris.RISOperations, 'get_server_capabilities')
    def test_get_server_capabilities_no_boot_modes_Gen9(
            self, cap_mock, nic_mock, gpu_mock,
            host_mock, mm_mock):
        str_val = mm_mock.return_value = '2.10'
        self.client.model = 'Gen9'
        nic_mock.return_value = None
        gpu_mock.return_value = None
        cap_mock.return_value = {'ilo_firmware_version': '2.10',
                                 'rom_firmware_version': 'x',
                                 'server_model': 'Gen9',
                                 'secure_boot': 'true'}
        capabilities = self.client.get_server_capabilities()
        cap_mock.assert_called_once_with()
        nic_mock.assert_called_once_with(self.client.ipmi_host_info, str_val)
        expected_capabilities = {'ilo_firmware_version': '2.10',
                                 'rom_firmware_version': 'x',
                                 'server_model': 'Gen9',
                                 'secure_boot': 'true'}
        self.assertEqual(expected_capabilities, capabilities)

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_activate_license(self, call_mock):
        self.client.activate_license('fake-key')
        call_mock.assert_called_once_with('activate_license', 'fake-key')

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_delete_raid_configuration(self, call_mock):
        self.client.delete_raid_configuration()
        call_mock.assert_called_once_with('delete_raid_configuration')

    @mock.patch.object(ris.RISOperations, 'get_product_name')
    def test_delete_raid_configuration_gen9(self, get_product_mock):
        self.client.model = 'Gen9'
        get_product_mock.return_value = 'ProLiant BL460c Gen9'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               '`delete_raid_configuration` is not supported '
                               'on ProLiant BL460c Gen9',
                               self.client.delete_raid_configuration)

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def test_delete_raid_configuration_gen8(self, get_product_mock):
        self.client.model = 'Gen8'
        get_product_mock.return_value = 'ProLiant DL380 G8'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               '`delete_raid_configuration` is not supported '
                               'on ProLiant DL380 G8',
                               self.client.delete_raid_configuration)

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_do_disk_erase(self, call_mock):
        self.client.do_disk_erase('SSD', None)
        call_mock.assert_called_once_with(
            'do_disk_erase', 'SSD', None)

    @mock.patch.object(ris.RISOperations, 'get_product_name')
    def test_do_disk_erase_gen9(self, get_product_mock):
        self.client.model = 'Gen9'
        get_product_mock.return_value = 'ProLiant BL460c Gen9'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               'The specified operation is not supported '
                               'on current platform.',
                               self.client.do_disk_erase,
                               'SSD', None)

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def test_do_disk_erase_gen8(self, get_product_mock):
        self.client.model = 'Gen8'
        get_product_mock.return_value = 'ProLiant DL380 G8'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               'The specified operation is not supported '
                               'on current platform.',
                               self.client.do_disk_erase,
                               'SSD', None)

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_do_one_button_secure_erase(self, call_mock):
        self.client.do_one_button_secure_erase()
        self.assertTrue(call_mock.called)

    @mock.patch.object(ris.RISOperations, 'get_product_name')
    def test_do_one_button_secure_erase_gen9(self, get_product_mock):
        self.client.model = 'Gen9'
        get_product_mock.return_value = 'ProLiant BL460c Gen9'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               'The specified operation is not supported '
                               'on current platform.',
                               self.client.do_one_button_secure_erase)

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def test_do_one_button_secure_erase_gen8(self, get_product_mock):
        self.client.model = 'Gen8'
        get_product_mock.return_value = 'ProLiant DL380 G8'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               'The specified operation is not supported '
                               'on current platform.',
                               self.client.do_one_button_secure_erase)

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_has_disk_erase_completed(self, call_mock):
        self.client.has_disk_erase_completed()
        call_mock.assert_called_once_with('has_disk_erase_completed')

    @mock.patch.object(ris.RISOperations, 'get_product_name')
    def test_has_disk_erase_completed_gen9(self, get_product_mock):
        self.client.model = 'Gen9'
        get_product_mock.return_value = 'ProLiant BL460c Gen9'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               'The specified operation is not supported '
                               'on current platform.',
                               self.client.has_disk_erase_completed)

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def test_has_disk_erase_completed_gen8(self, get_product_mock):
        self.client.model = 'Gen8'
        get_product_mock.return_value = 'ProLiant DL380 G8'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               'The specified operation is not supported '
                               'on current platform.',
                               self.client.has_disk_erase_completed)

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_create_raid_configuration(self, call_mock):
        ld1 = {"size_gb": 150, "raid_level": '0', "is_root_volume": True}
        raid_config = {"logical_disks": [ld1]}
        self.client.create_raid_configuration(raid_config)
        call_mock.assert_called_once_with('create_raid_configuration',
                                          raid_config)

    @mock.patch.object(ris.RISOperations, 'get_product_name')
    def test_create_raid_configuration_gen9(self, get_product_mock):
        self.client.model = 'Gen9'
        ld1 = {"size_gb": 150, "raid_level": '0', "is_root_volume": True}
        raid_config = {"logical_disks": [ld1]}
        get_product_mock.return_value = 'ProLiant BL460c Gen9'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               '`create_raid_configuration` is not supported '
                               'on ProLiant BL460c Gen9',
                               self.client.create_raid_configuration,
                               raid_config)

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def test_create_raid_configuration_gen8(self, get_product_mock):
        self.client.model = 'Gen8'
        ld1 = {"size_gb": 150, "raid_level": '0', "is_root_volume": True}
        raid_config = {"logical_disks": [ld1]}
        get_product_mock.return_value = 'ProLiant DL380 G8'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               '`create_raid_configuration` is not supported '
                               'on ProLiant DL380 G8',
                               self.client.create_raid_configuration,
                               raid_config)

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_read_raid_configuration(self, call_mock):
        ld1 = {"size_gb": 150, "raid_level": '0', "is_root_volume": True}
        raid_config = {"logical_disks": [ld1]}
        self.client.read_raid_configuration(raid_config)
        call_mock.assert_called_once_with('read_raid_configuration',
                                          raid_config)

    @mock.patch.object(ris.RISOperations, 'get_product_name')
    def test_read_raid_configuration_gen9(self, get_product_mock):
        self.client.model = 'Gen9'
        ld1 = {"size_gb": 150, "raid_level": '0', "is_root_volume": True}
        raid_config = {"logical_disks": [ld1]}
        get_product_mock.return_value = 'ProLiant BL460c Gen9'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               '`read_raid_configuration` is not supported '
                               'on ProLiant BL460c Gen9',
                               self.client.read_raid_configuration,
                               raid_config)

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def test_read_raid_configuration_gen8(self, get_product_mock):
        self.client.model = 'Gen8'
        ld1 = {"size_gb": 150, "raid_level": '0', "is_root_volume": True}
        raid_config = {"logical_disks": [ld1]}
        get_product_mock.return_value = 'ProLiant DL380 G8'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               '`read_raid_configuration` is not supported '
                               'on ProLiant DL380 G8',
                               self.client.read_raid_configuration,
                               raid_config)

    @mock.patch.object(ris.RISOperations, 'eject_virtual_media')
    def test_eject_virtual_media_gen9(self, eject_virtual_media_mock):
        self.client.model = 'Gen9'
        self.client.eject_virtual_media(device='FLOPPY')
        eject_virtual_media_mock.assert_called_once_with('FLOPPY')

    @mock.patch.object(ribcl.RIBCLOperations, 'eject_virtual_media')
    def test_eject_virtual_media_gen8(self, eject_virtual_media_mock):
        self.client.model = 'Gen8'
        self.client.eject_virtual_media(device='FLOPPY')
        eject_virtual_media_mock.assert_called_once_with('FLOPPY')

    @mock.patch.object(ris.RISOperations, 'get_vm_status')
    def test_get_vm_status_gen9(self, get_vm_status_mock):
        self.client.model = 'Gen9'
        self.client.get_vm_status(device='FLOPPY')
        get_vm_status_mock.assert_called_once_with('FLOPPY')

    @mock.patch.object(ribcl.RIBCLOperations, 'get_vm_status')
    def test_get_vm_status_gen8(self, get_vm_status_mock):
        self.client.model = 'Gen8'
        self.client.get_vm_status(device='FLOPPY')
        get_vm_status_mock.assert_called_once_with('FLOPPY')

    @mock.patch.object(ris.RISOperations, 'set_vm_status')
    def test_set_vm_status_gen9(self, set_vm_status_mock):
        self.client.model = 'Gen9'
        self.client.set_vm_status(device='FLOPPY', boot_option='BOOT_ONCE',
                                  write_protect='YES')
        set_vm_status_mock.assert_called_once_with('FLOPPY', 'BOOT_ONCE',
                                                   'YES')

    @mock.patch.object(ribcl.RIBCLOperations, 'set_vm_status')
    def test_set_vm_status_gen8(self, set_vm_status_mock):
        self.client.model = 'Gen8'
        self.client.set_vm_status(device='FLOPPY', boot_option='BOOT_ONCE',
                                  write_protect='YES')
        set_vm_status_mock.assert_called_once_with('FLOPPY', 'BOOT_ONCE',
                                                   'YES')

    @mock.patch.object(ris.RISOperations, 'insert_virtual_media')
    def test_insert_virtual_media_gen9(self, insert_virtual_media_mock):
        self.client.model = 'Gen9'
        self.client.insert_virtual_media(url="http://ilo/fpy.iso",
                                         device='FLOPPY')
        insert_virtual_media_mock.assert_called_once_with("http://ilo/fpy.iso",
                                                          "FLOPPY")

    @mock.patch.object(ribcl.RIBCLOperations, 'insert_virtual_media')
    def test_insert_virtual_media_gen8(self, insert_virtual_media_mock):
        self.client.model = 'Gen8'
        self.client.insert_virtual_media(url="http://ilo/fpy.iso",
                                         device='FLOPPY')
        insert_virtual_media_mock.assert_called_once_with("http://ilo/fpy.iso",
                                                          "FLOPPY")

    @mock.patch.object(ris.RISOperations, 'get_one_time_boot')
    def test_get_one_time_boot_gen9(self, get_one_time_boot_mock):
        self.client.model = 'Gen9'
        self.client.get_one_time_boot()
        get_one_time_boot_mock.assert_called_once_with()

    @mock.patch.object(ribcl.RIBCLOperations, 'get_one_time_boot')
    def test_get_one_time_boot_gen8(self, get_one_time_boot_mock):
        self.client.model = 'Gen8'
        self.client.get_one_time_boot()
        get_one_time_boot_mock.assert_called_once_with()

    @mock.patch.object(ris.RISOperations, 'set_one_time_boot')
    def test_set_one_time_boot_gen9(self, set_one_time_boot_mock):
        self.client.model = 'Gen9'
        self.client.set_one_time_boot('cdrom')
        set_one_time_boot_mock.assert_called_once_with('cdrom')

    @mock.patch.object(ribcl.RIBCLOperations, 'set_one_time_boot')
    def test_set_one_time_boot_gen8(self, set_one_time_boot_mock):
        self.client.model = 'Gen8'
        self.client.set_one_time_boot('cdrom')
        set_one_time_boot_mock.assert_called_once_with('cdrom')

    @mock.patch.object(ris.RISOperations, 'update_persistent_boot')
    def test_update_persistent_boot_gen9(self, update_persistent_boot_mock):
        self.client.model = 'Gen9'
        self.client.update_persistent_boot(['cdrom'])
        update_persistent_boot_mock.assert_called_once_with(['cdrom'])

    @mock.patch.object(ribcl.RIBCLOperations, 'update_persistent_boot')
    def test_update_persistent_boot_gen8(self, update_persistent_boot_mock):
        self.client.model = 'Gen8'
        self.client.update_persistent_boot(['cdrom'])
        update_persistent_boot_mock.assert_called_once_with(['cdrom'])

    @mock.patch.object(ris.RISOperations, 'get_persistent_boot_device')
    def test_get_persistent_boot_device_gen9(self, get_pers_boot_device_mock):
        self.client.model = 'Gen9'
        self.client.get_persistent_boot_device()
        get_pers_boot_device_mock.assert_called_once_with()

    @mock.patch.object(ribcl.RIBCLOperations, 'get_persistent_boot_device')
    def test_get_persistent_boot_device_gen8(self, get_pers_boot_device_mock):
        self.client.model = 'Gen8'
        self.client.get_persistent_boot_device()
        get_pers_boot_device_mock.assert_called_once_with()

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_update_firmware(self, _call_method_mock):
        # | GIVEN |
        some_url = 'some-url'
        some_component_type = 'ilo'
        # | WHEN |
        self.client.update_firmware(some_url, some_component_type)
        # | THEN |
        _call_method_mock.assert_called_once_with('update_firmware',
                                                  some_url,
                                                  some_component_type)

    @mock.patch.object(ris.RISOperations, 'hold_pwr_btn')
    def test_hold_pwr_btn_gen9(self, hold_pwr_btn_mock):
        self.client.model = 'Gen9'
        self.client.hold_pwr_btn()
        self.assertTrue(hold_pwr_btn_mock.called)

    @mock.patch.object(ribcl.RIBCLOperations, 'hold_pwr_btn')
    def test_hold_pwr_btn_gen8(self, hold_pwr_btn_mock):
        self.client.model = 'Gen8'
        self.client.hold_pwr_btn()
        self.assertTrue(hold_pwr_btn_mock.called)

    @mock.patch.object(ris.RISOperations, 'set_host_power')
    def test_set_host_power_gen9(self, set_host_power_mock):
        self.client.model = 'Gen9'
        self.client.set_host_power('ON')
        set_host_power_mock.assert_called_once_with('ON')

    @mock.patch.object(ribcl.RIBCLOperations, 'set_host_power')
    def test_set_host_power_gen8(self, set_host_power_mock):
        self.client.model = 'Gen8'
        self.client.set_host_power('ON')
        set_host_power_mock.assert_called_once_with('ON')

    @mock.patch.object(ris.RISOperations, 'press_pwr_btn')
    def test_press_pwr_btn_gen9(self, press_pwr_btn_mock):
        self.client.model = 'Gen9'
        self.client.press_pwr_btn()
        self.assertTrue(press_pwr_btn_mock.called)

    @mock.patch.object(ribcl.RIBCLOperations, 'press_pwr_btn')
    def test_press_pwr_btn_gen8(self, press_pwr_btn_mock):
        self.client.model = 'Gen8'
        self.client.press_pwr_btn()
        self.assertTrue(press_pwr_btn_mock.called)

    @mock.patch.object(ris.RISOperations, 'reset_server')
    def test_reset_server_gen9(self, reset_server_mock):
        self.client.model = 'Gen9'
        self.client.reset_server()
        self.assertTrue(reset_server_mock.called)

    @mock.patch.object(ribcl.RIBCLOperations, 'reset_server')
    def test_reset_server_gen8(self, reset_server_mock):
        self.client.model = 'Gen8'
        self.client.reset_server()
        self.assertTrue(reset_server_mock.called)

    @mock.patch.object(ris.RISOperations, 'get_current_bios_settings')
    def test_get_current_bios_settings_gen9(self, cur_bios_settings_mock):
        self.client.model = 'Gen9'
        apply_filter = True
        self.client.get_current_bios_settings(apply_filter)
        cur_bios_settings_mock.assert_called_once_with(True)

    @mock.patch.object(ris.RISOperations, 'get_pending_bios_settings')
    def test_get_pending_bios_settings_gen9(self, pending_bios_settings_mock):
        self.client.model = 'Gen9'
        apply_filter = True
        self.client.get_pending_bios_settings(apply_filter)
        pending_bios_settings_mock.assert_called_once_with(True)

    @mock.patch.object(ris.RISOperations, 'get_default_bios_settings')
    def test_get_default_bios_settings_gen9(self, def_bios_settings_mock):
        self.client.model = 'Gen9'
        apply_filter = False
        self.client.get_default_bios_settings(apply_filter)
        def_bios_settings_mock.assert_called_once_with(False)

    @mock.patch.object(ris.RISOperations, 'set_bios_settings')
    def test_set_bios_settings_gen9(self, bios_settings_mock):
        self.client.model = 'Gen9'
        apply_filter = False
        d = {'a': 'blah', 'b': 'blah blah'}
        self.client.set_bios_settings(d, apply_filter)
        bios_settings_mock.assert_called_once_with(d, False)

    @mock.patch.object(client.IloClient.cls, '_call_method')
    @mock.patch.object(snmp_cpqdisk_sizes, 'get_local_gb')
    def test_get_essential_prop_no_snmp_ris(self,
                                            snmp_mock,
                                            call_mock):
        self.client.model = 'Gen9'
        properties = {'local_gb': 250}
        data = {'properties': properties}
        call_mock.return_value = data
        self.client.get_essential_properties()
        call_mock.assert_called_once_with('get_essential_properties')
        self.assertFalse(snmp_mock.called)

    @mock.patch.object(client.IloClient.cls, '_call_method')
    @mock.patch.object(snmp_cpqdisk_sizes, 'get_local_gb')
    def test_get_essential_prop_no_snmp_local_gb_0(self,
                                                   snmp_mock,
                                                   call_mock):
        self.client.model = 'Gen9'
        properties = {'local_gb': 0}
        data = {'properties': properties}
        call_mock.return_value = data
        self.client.get_essential_properties()
        call_mock.assert_called_once_with('get_essential_properties')
        self.assertFalse(snmp_mock.called)

    @mock.patch.object(client.IloClient.cls, '_call_method')
    @mock.patch.object(snmp_cpqdisk_sizes, 'get_local_gb')
    def test_get_essential_prop_snmp_true(self,
                                          snmp_mock,
                                          call_mock):
        self.client.model = 'Gen9'
        snmp_credentials = {'auth_user': 'user',
                            'auth_prot_pp': '1234',
                            'auth_priv_pp': '4321',
                            'auth_protocol': 'SHA',
                            'priv_protocol': 'AES',
                            'snmp_inspection': 'true'}
        self.client.snmp_credentials = snmp_credentials
        properties = {'local_gb': 0}
        data = {'properties': properties}
        call_mock.return_value = data
        snmp_mock.return_value = 250
        self.client.get_essential_properties()
        call_mock.assert_called_once_with('get_essential_properties')
        snmp_mock.assert_called_once_with(
            self.client.ipmi_host_info['address'], snmp_credentials)

    @mock.patch.object(client.IloClient.cls, '_call_method')
    @mock.patch.object(snmp_cpqdisk_sizes, 'get_local_gb')
    def test_get_essential_prop_snmp_true_local_gb_0(self,
                                                     snmp_mock,
                                                     call_mock):
        self.client.model = 'Gen9'
        snmp_credentials = {'auth_user': 'user',
                            'auth_prot_pp': '1234',
                            'auth_priv_pp': '4321',
                            'auth_protocol': 'SHA',
                            'priv_protocol': 'AES',
                            'snmp_inspection': 'true'}
        self.client.snmp_credentials = snmp_credentials
        properties = {'local_gb': 0}
        data = {'properties': properties}
        call_mock.return_value = data
        snmp_mock.return_value = 0
        self.client.get_essential_properties()
        call_mock.assert_called_once_with('get_essential_properties')
        snmp_mock.assert_called_once_with(
            self.client.ipmi_host_info['address'], snmp_credentials)

    @mock.patch.object(snmp_cpqdisk_sizes, 'get_local_gb')
    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_essential_prop_snmp_false_local_gb_0(self, call_mock,
                                                      snmp_mock):
        self.client.model = 'Gen9'
        snmp_credentials = {'auth_user': 'user',
                            'auth_prot_pp': '1234',
                            'auth_priv_pp': '4321',
                            'auth_protocol': 'SHA',
                            'priv_protocol': 'AES',
                            'snmp_inspection': False}
        self.client.snmp_inspection = False
        self.client.snmp_credentials = snmp_credentials
        properties = {'local_gb': 0}
        data = {'properties': properties}
        call_mock.return_value = data
        self.client.get_essential_properties()
        call_mock.assert_called_once_with('get_essential_properties')
        self.assertFalse(snmp_mock.called)

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_inject_nmi(self, call_mock):
        self.client.inject_nmi()
        call_mock.assert_called_once_with('inject_nmi')

    @mock.patch.object(ris.RISOperations, 'inject_nmi')
    def test_inject_nmi_gen9(self, inject_nmi_mock):
        self.client.model = 'Gen9'
        self.client.inject_nmi()
        inject_nmi_mock.assert_called_once_with()

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def test_inject_nmi_gen8(self, product_mock):
        self.client.model = 'Gen8'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               'not supported',
                               self.client.inject_nmi)

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_host_post_state(self, call_mock):
        self.client.get_host_post_state()
        call_mock.assert_called_once_with('get_host_post_state')

    @mock.patch.object(ris.RISOperations, 'get_host_post_state')
    def test_get_host_post_state_gen9(self, get_host_post_state_mock):
        self.client.model = 'Gen9'
        self.client.get_host_post_state()
        get_host_post_state_mock.assert_called_once_with()

    @mock.patch.object(ribcl.RIBCLOperations, 'get_host_post_state')
    def test_get_host_post_state_gen8(self, get_host_post_state_mock):
        self.client.model = 'Gen8'
        self.client.get_host_post_state()
        get_host_post_state_mock.assert_called_once_with()

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_bios_settings_result(self, call_mock):
        self.client.get_bios_settings_result()
        call_mock.assert_called_once_with('get_bios_settings_result')

    @mock.patch.object(ris.RISOperations, 'get_bios_settings_result')
    def test_get_bios_settings_result_gen9(self, get_bios_settings_mock):
        self.client.model = 'Gen9'
        self.client.get_bios_settings_result()
        get_bios_settings_mock.assert_called_once_with()

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def test_get_bios_settings_result_gen8(self, product_mock):
        self.client.model = 'Gen8'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               'not supported',
                               self.client.get_bios_settings_result)

    @mock.patch.object(client.IloClient.cls, '_call_method')
    def test_get_available_disk_types(self, call_mock):
        self.client.get_available_disk_types()
        call_mock.assert_called_once_with('get_available_disk_types')

    @mock.patch.object(ris.RISOperations, 'get_product_name')
    def test_get_available_disk_types_gen9(self, get_product_mock):
        self.client.model = 'Gen9'
        get_product_mock.return_value = 'ProLiant BL460c Gen9'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               'not supported',
                               self.client.get_available_disk_types)

    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def test_get_available_disk_types_gen8(self, get_product_mock):
        self.client.model = 'Gen8'
        get_product_mock.return_value = 'ProLiant DL380 G8'
        self.assertRaisesRegex(exception.IloCommandNotSupportedError,
                               'not supported',
                               self.client.get_available_disk_types)


class IloRedfishClientTestCase(testtools.TestCase):

    @mock.patch.object(redfish, 'RedfishOperations',
                       spec_set=True, autospec=True)
    @mock.patch.object(ribcl.RIBCLOperations, 'get_product_name')
    def setUp(self, product_mock, redfish_mock):
        super(IloRedfishClientTestCase, self).setUp()
        self.redfish_mock = redfish_mock
        product_mock.return_value = 'Gen10'
        self.client = client.IloClient.cls("1.2.3.4", "Admin", "admin")

    def test_calling_redfish_operations_gen10(self):
        self.client.model = 'Gen10'

        def validate_method_calls(methods, method_args, missed_ops):
            for redfish_method_name in methods:
                try:
                    if method_args:
                        eval('self.client.' + redfish_method_name)(
                            *method_args)
                    else:
                        eval('self.client.' + redfish_method_name)()
                    if redfish_method_name not in ('unset_iscsi_boot_info',
                                                   'set_iscsi_boot_info'):
                        self.assertTrue(eval(
                            'self.redfish_mock.return_value.'
                            + redfish_method_name).called)
                    validate_method_calls.no_test_cases += 1
                except TypeError:
                    missed_ops.append(redfish_method_name)

        validate_method_calls.no_test_cases = 0
        missed_operations = []
        validate_method_calls(
            client.SUPPORTED_REDFISH_METHODS, (), missed_operations)

        more_missed_operations = []
        validate_method_calls(
            missed_operations, ('arg',), more_missed_operations)

        even_more_missed_operations = []
        validate_method_calls(
            more_missed_operations, ('arg1', 'arg2'),
            even_more_missed_operations)
        if (len(even_more_missed_operations) == 1):
            self.assertEqual('set_iscsi_info',
                             even_more_missed_operations[0])
        else:
            self.assertEqual(2, len(even_more_missed_operations))
            self.assertEqual(len(client.SUPPORTED_REDFISH_METHODS) - 2,
                             validate_method_calls.no_test_cases)



# Copyright 2017 Hewlett Packard Enterprise Development LP
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sushy.resources import base
from sushy import utils as sushy_utils

from proliantutils.redfish import utils


class Volume(base.ResourceBase):
    """This class represents the Volume resource"""

    identity = base.Field('Id', required=True)
    """The processor identity string"""

    capacity_bytes = base.Field('CapacityBytes', adapter=int)
    """The size in bytes of this Volume"""


class VolumeCollection(base.ResourceCollectionBase):
    """This class represents the collection of Volume resource"""

    @property
    def _resource_type(self):
        return Volume

    @property
    @sushy_utils.cache_it
    def maximum_size_bytes(self):
        """Gets the biggest volume

        :returns size in bytes.
        """
        return utils.max_safe([member.capacity_bytes
                               for member in self.get_members()])



# Copyright 2017-2022 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from sushy.resources import base
from sushy import utils as sushy_utils

LOG = logging.getLogger(__name__)

CLASSCODE_FOR_GPU_DEVICES = [3]
SUBCLASSCODE_FOR_GPU_DEVICES = [0, 1, 2, 128]


class PCIDevice(base.ResourceBase):

    identity = base.Field('Id', required=True)

    name = base.Field('Name')

    class_code = base.Field('ClassCode')

    sub_class_code = base.Field('SubclassCode')

    odata_id = base.Field('@odata.id')

    vendor_id = base.Field('VendorID')

    @property
    @sushy_utils.cache_it
    def nic_capacity(self):
        if self.name:
            for item in self.name.split():
                if 'Gb' in item:
                    capacity = item.strip('Gb')
                    return int(capacity) if capacity.isdigit() else 0
        return 0


class PCIDeviceCollection(base.ResourceCollectionBase):

    @property
    def _resource_type(self):
        return PCIDevice

    @property
    @sushy_utils.cache_it
    def gpu_devices(self):
        """Returns the list of gpu devices uris"""
        gpu_devices = []
        for member in self.get_members():
            if member.class_code in CLASSCODE_FOR_GPU_DEVICES:
                if member.sub_class_code in SUBCLASSCODE_FOR_GPU_DEVICES:
                    gpu_devices.append(member.odata_id)
        return gpu_devices

    @property
    @sushy_utils.cache_it
    def max_nic_capacity(self):
        """Gets the maximum NIC capacity"""
        return str(max([m.nic_capacity for m in self.get_members()])) + 'Gb'

    @property
    @sushy_utils.cache_it
    def vendor_id(self):
        """Returns the vendor_id dictionary"""
        vendor_id = {}
        for member in self.get_members():
            if member.odata_id in self.gpu_devices:
                vendor_id.update({member.identity: member.vendor_id})
        return vendor_id



# Copyright 2017 Hewlett Packard Enterprise Development LP
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
from unittest import mock

import testtools

from proliantutils.redfish.resources.system import constants
from proliantutils.redfish.resources.system import memory


class MemoryTestCase(testtools.TestCase):

    def setUp(self):
        super(MemoryTestCase, self).setUp()
        self.conn = mock.MagicMock()
        with open('proliantutils/tests/redfish/'
                  'json_samples/logical_nvdimm.json', 'r') as f:
            self.conn.get.return_value.json.return_value = (
                json.loads(f.read())['logical_nvdimm'])

        self.mem_inst = memory.Memory(
            self.conn, '/redfish/v1/Systems/1/Memory/proc1logicalnvdimm1/',
            redfish_version='1.0.2')

    def test_attributes(self):
        self.assertEqual('proc1logicalnvdimm1',
                         self.mem_inst.identity)
        self.assertEqual('proc1logicalnvdimm1',
                         self.mem_inst.name)
        self.assertEqual(constants.MEMORY_DEVICE_TYPE_LOGICAL,
                         self.mem_inst.memory_device_type)
        self.assertEqual(constants.MEMORY_TYPE_NVDIMM_N,
                         self.mem_inst.memory_type)


class MemoryCollectionTestCase(testtools.TestCase):

    def setUp(self):
        super(MemoryCollectionTestCase, self).setUp()
        self.conn = mock.MagicMock()
        with open('proliantutils/tests/redfish/'
                  'json_samples/memory_collection.json', 'r') as f:
            self.conn.get.return_value.json.return_value = (
                json.loads(f.read()))

        self.mem_col_inst = memory.MemoryCollection(
            self.conn, '/redfish/v1/Systems/1/Memory/',
            redfish_version='1.0.2')

    def test_details_nvdimm_n(self):
        with open('proliantutils/tests/redfish/'
                  'json_samples/logical_nvdimm.json', 'r') as f:
            mem1 = (
                json.loads(f.read())['default'])
        self.conn.get.return_value.json.return_value = mem1
        ret = self.mem_col_inst.details()
        self.assertEqual(ret.has_persistent_memory, True)
        self.assertEqual(ret.has_nvdimm_n, True)
        self.assertEqual(ret.has_logical_nvdimm_n, False)

    def test_details_logical(self):
        with open('proliantutils/tests/redfish/'
                  'json_samples/logical_nvdimm.json', 'r') as f:
            mem1 = (
                json.loads(f.read())['logical_nvdimm'])
        self.conn.get.return_value.json.return_value = mem1
        ret = self.mem_col_inst.details()
        self.assertEqual(ret.has_persistent_memory, True)
        self.assertEqual(ret.has_nvdimm_n, True)
        self.assertEqual(ret.has_logical_nvdimm_n, True)



# Copyright 2017 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

__author__ = 'HPE'

import sushy

from proliantutils import exception
from proliantutils import log
from proliantutils.redfish import utils

LOG = log.get_logger(__name__)


def _get_attribute_value_of(resource, attribute_name, default=None):
    """Gets the value of attribute_name from the resource

    It catches the exception, if any, while retrieving the
    value of attribute_name from resource and returns default.

    :param resource: The resource object
    :attribute_name: Property of the resource
    :returns the property value if no error encountered
        else return 0.
    """
    try:
        return getattr(resource, attribute_name)
    except (sushy.exceptions.SushyError,
            exception.MissingAttributeError) as e:
        msg = (('The Redfish controller failed to get the '
                'attribute %(attribute)s from resource %(resource)s. '
                'Error %(error)s') % {'error': str(e),
                                      'attribute': attribute_name,
                                      'resource':
                                      resource.__class__.__name__})
        LOG.debug(msg)
        return default


def get_local_gb(system_obj):
    """Gets the largest volume or the largest disk

    :param system_obj: The HPESystem object.
    :returns the size in GB
    """
    local_max_bytes = 0
    logical_max_mib = 0
    volume_max_bytes = 0
    physical_max_mib = 0
    drives_max_bytes = 0
    simple_max_bytes = 0

    # Gets the resources and properties
    # its quite possible for a system to lack the resource, hence its
    # URI may also be lacking.

    # Check if smart_storage resource exist at the system
    smart_resource = _get_attribute_value_of(system_obj, 'smart_storage')
    # Check if storage resource exist at the system
    storage_resource = _get_attribute_value_of(system_obj, 'storages')

    if smart_resource is not None:
        logical_max_mib = _get_attribute_value_of(
            smart_resource, 'logical_drives_maximum_size_mib', default=0)
    if storage_resource is not None:
        volume_max_bytes = _get_attribute_value_of(
            storage_resource, 'volumes_maximum_size_bytes', default=0)

    # Get the largest volume from the system.
    local_max_bytes = utils.max_safe([(logical_max_mib * 1024 * 1024),
                                     volume_max_bytes])
    # if volume is not found, then traverse through the possible disk drives
    # and get the biggest disk.
    if local_max_bytes == 0:
        if smart_resource is not None:
            physical_max_mib = _get_attribute_value_of(
                smart_resource, 'physical_drives_maximum_size_mib', default=0)

        if storage_resource is not None:
            drives_max_bytes = _get_attribute_value_of(
                storage_resource, 'drives_maximum_size_bytes', default=0)

        # Check if the SimpleStorage resource exist at the system.
        simple_resource = _get_attribute_value_of(system_obj,
                                                  'simple_storages')
        if simple_resource is not None:
            simple_max_bytes = _get_attribute_value_of(
                simple_resource, 'maximum_size_bytes', default=0)

        local_max_bytes = utils.max_safe([(physical_max_mib * 1024 * 1024),
                                         drives_max_bytes, simple_max_bytes])
    # Convert the received size to GB and reduce the value by 1 Gb as
    # ironic requires the local_gb to be returned 1 less than actual size.
    local_gb = 0
    if local_max_bytes > 0:
        local_gb = int(local_max_bytes / (1024 * 1024 * 1024)) - 1
    else:
        msg = ('The maximum size for the hard disk or logical '
               'volume could not be determined.')
        LOG.debug(msg)
    return local_gb


def has_ssd(system_obj):
    """Gets if the system has any drive as SSD drive

    :param system_obj: The HPESystem object.
    :returns True if system has SSD drives.
    """
    smart_value = False
    storage_value = False
    smart_resource = _get_attribute_value_of(system_obj, 'smart_storage')
    if smart_resource is not None:
        smart_value = _get_attribute_value_of(
            smart_resource, 'has_ssd', default=False)

    if smart_value:
        return smart_value

    # Its returned before just to avoid hitting BMC if we have
    # already got the SSD device above.
    storage_resource = _get_attribute_value_of(system_obj, 'storages')
    if storage_resource is not None:
        storage_value = _get_attribute_value_of(
            storage_resource, 'has_ssd', default=False)
    return storage_value


def has_rotational(system_obj):
    """Gets if the system has any drive as HDD drive

    :param system_obj: The HPESystem object.
    :returns True if system has HDD drives.
    """
    smart_value = False
    storage_value = False
    smart_resource = _get_attribute_value_of(system_obj, 'smart_storage')
    if smart_resource is not None:
        smart_value = _get_attribute_value_of(
            smart_resource, 'has_rotational', default=False)

    if smart_value:
        return smart_value

    # Its returned before just to avoid hitting BMC if we have
    # already got the HDD device above.
    storage_resource = _get_attribute_value_of(system_obj, 'storages')
    if storage_resource is not None:
        storage_value = _get_attribute_value_of(
            storage_resource, 'has_rotational', default=False)
    return storage_value


def has_nvme_ssd(system_obj):
    """Gets if the system has any drive as NVMe SSD drive

    :param system_obj: The HPESystem object.
    :returns True if system has SSD drives and protocol is NVMe.
    """
    storage_value = False
    storage_resource = _get_attribute_value_of(system_obj, 'storages')
    if storage_resource is not None:
        storage_value = _get_attribute_value_of(
            storage_resource, 'has_nvme_ssd', default=False)

    return storage_value


def get_drive_rotational_speed_rpm(system_obj):
    """Gets the set of rotational speed rpms of the disks.

    :param system_obj: The HPESystem object.
    :returns the set of rotational speed rpms of the HDD devices.
    """
    speed = set()
    smart_resource = _get_attribute_value_of(system_obj, 'smart_storage')
    if smart_resource is not None:
        speed.update(_get_attribute_value_of(
            smart_resource, 'drive_rotational_speed_rpm', default=set()))
    storage_resource = _get_attribute_value_of(system_obj, 'storages')
    if storage_resource is not None:
        speed.update(_get_attribute_value_of(
            storage_resource, 'drive_rotational_speed_rpm', default=set()))
    return speed



# Copyright 2014 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Test class for IPMI Module."""

import subprocess
import unittest
from unittest import mock

from proliantutils.ilo import ipmi
from proliantutils.tests.ilo import ipmi_sample_outputs as constants


class IloIpmiTestCase(unittest.TestCase):

    def setUp(self):
        super(IloIpmiTestCase, self).setUp()
        self.info = {'address': "x.x.x.x",
                     'username': "admin",
                     'password': "Admin"}

    def test_get_ilo_version_valid_1(self):
        in_vals = '2.03'
        expected = 2.03
        actual = ipmi.get_ilo_version(in_vals)
        self.assertEqual(actual, expected)

    def test_get_ilo_version_valid_2(self):
        in_vals = '2.3'
        expected = 2.30
        actual = ipmi.get_ilo_version(in_vals)
        self.assertEqual(actual, expected)

    def test_get_ilo_version_invalid(self):
        in_vals = 'x.y'
        expected = None
        actual = ipmi.get_ilo_version(in_vals)
        self.assertEqual(actual, expected)

    def test_get_ilo_version_appended_spaces(self):
        in_vals = ' 2.50 '
        expected = 2.5
        actual = ipmi.get_ilo_version(in_vals)
        self.assertEqual(actual, expected)

    def test_get_ilo_version_none(self):
        in_vals = None
        expected = None
        actual = ipmi.get_ilo_version(in_vals)
        self.assertEqual(actual, expected)

    @mock.patch.object(ipmi, '_parse_ipmi_nic_capacity')
    @mock.patch.object(ipmi, '_exec_ipmitool')
    def test_get_nic_capacity_fw_lt_suggested(self, ipmi_mock, parse_mock):
        fw_rev = constants.LESSER_THAN_MIN_SUGGESTED_FW_STR
        ipmi_mock.return_value = constants.NIC_FRU_OUT
        parse_mock.return_value = "1Gb"
        expected_out = "1Gb"
        actual_out = ipmi.get_nic_capacity(self.info, fw_rev)
        self.assertEqual(expected_out, actual_out)
        ipmi_mock.assert_called_once_with(mock.ANY, "fru print 0x0")

    @mock.patch.object(ipmi, '_parse_ipmi_nic_capacity')
    @mock.patch.object(ipmi, '_exec_ipmitool')
    def test_get_nic_capacity_fw_lt_suggested_none(self,
                                                   ipmi_mock,
                                                   parse_mock):
        fw_rev = constants.LESSER_THAN_MIN_SUGGESTED_FW_STR
        ipmi_mock.return_value = constants.NIC_FRU_OUT
        parse_mock.return_value = None
        actual_out = ipmi.get_nic_capacity(self.info, fw_rev)
        self.assertIsNone(actual_out)
        self.assertEqual(ipmi_mock.call_count, 126)

    @mock.patch.object(ipmi, '_parse_ipmi_nic_capacity')
    @mock.patch.object(ipmi, '_exec_ipmitool')
    def test_get_nic_capacity_fw_lt_suggested_out_of_range_check(
            self, ipmi_mock, parse_mock):
        fw_rev = constants.LESSER_THAN_MIN_SUGGESTED_FW_STR
        actual_out = ipmi.get_nic_capacity(self.info, fw_rev)
        format_call_args = map(lambda x: x[0][1],
                               ipmi_mock.call_args_list)
        self.assertNotIn("fru print 0x7d", format_call_args)
        self.assertEqual(actual_out, None)

    @mock.patch.object(ipmi, '_parse_ipmi_nic_capacity')
    @mock.patch.object(ipmi, '_exec_ipmitool')
    def test_get_nic_capacity_fw_lt_suggested_in_range_check(
            self, ipmi_mock, parse_mock):
        fw_rev = constants.LESSER_THAN_MIN_SUGGESTED_FW_STR
        actual_out = ipmi.get_nic_capacity(self.info, fw_rev)
        format_call_args = map(lambda x: x[0][1],
                               ipmi_mock.call_args_list)
        self.assertIn("fru print 0xef", format_call_args)
        self.assertEqual(actual_out, None)

    @mock.patch.object(ipmi, '_parse_ipmi_nic_capacity')
    @mock.patch.object(ipmi, '_exec_ipmitool')
    def test_get_nic_capacity_fw_suggested(self, ipmi_mock, parse_mock):
        fw_rev = constants.MIN_SUGGESTED_FW_STR
        ipmi_mock.return_value = constants.NIC_FRU_OUT_ALL
        parse_mock.return_value = "1Gb"
        expected_out = "1Gb"
        actual_out = ipmi.get_nic_capacity(self.info, fw_rev)
        self.assertEqual(expected_out, actual_out)
        ipmi_mock.assert_called_once_with(mock.ANY, "fru print")
        self.assertEqual(ipmi_mock.call_count, 1)

    @mock.patch.object(ipmi, '_parse_ipmi_nic_capacity')
    @mock.patch.object(ipmi, '_exec_ipmitool')
    def test_get_nic_capacity_fw_suggested_none(self, ipmi_mock, parse_mock):
        fw_rev = constants.MIN_SUGGESTED_FW_STR
        ipmi_mock.return_value = constants.NIC_FRU_OUT_ALL
        parse_mock.return_value = None
        actual_out = ipmi.get_nic_capacity(self.info, fw_rev)
        self.assertIsNone(actual_out)
        self.assertEqual(ipmi_mock.call_count, 1)

    @mock.patch.object(ipmi, '_parse_ipmi_nic_capacity')
    @mock.patch.object(ipmi, '_exec_ipmitool')
    def test_get_nic_capacity_fw_gt_suggested(self, ipmi_mock, parse_mock):
        fw_rev = constants.GREATER_THAN_MIN_SUGGESTED_FW_STR
        ipmi_mock.return_value = constants.NIC_FRU_OUT_ALL
        parse_mock.return_value = "1Gb"
        expected_out = "1Gb"
        actual_out = ipmi.get_nic_capacity(self.info, fw_rev)
        self.assertEqual(expected_out, actual_out)
        ipmi_mock.assert_called_once_with(mock.ANY, "fru print")

    @mock.patch.object(ipmi, '_parse_ipmi_nic_capacity')
    @mock.patch.object(ipmi, '_exec_ipmitool')
    def test_get_nic_capacity_fw_lt_suggested_loop_N_times(self,
                                                           ipmi_mock,
                                                           parse_mock):
        fw_rev = constants.LESSER_THAN_MIN_SUGGESTED_FW_STR
        ipmi_mock.side_effect = ["Device not present", "Device not present",
                                 "Device not present", "Device not present",
                                 "Device not present", "Device not present",
                                 "Device not present", "Device not present",
                                 constants.NIC_FRU_OUT]
        parse_mock.return_value = "1Gb"
        expected_out = "1Gb"
        actual_out = ipmi.get_nic_capacity(self.info, fw_rev)
        self.assertEqual(ipmi_mock.call_count, 9)
        self.assertEqual(expected_out, actual_out)

    @mock.patch.object(subprocess, 'Popen')
    def test__exec_ipmitool(self, popen_mock):
        pro_obj = mock.MagicMock()
        popen_mock.return_value = pro_obj
        pro_obj.communicate.return_value = constants.NIC_FRU_OUT_TUPLE
        expected_output = constants.NIC_FRU_OUT
        cmd = "fru print 0x64"
        actual_out = ipmi._exec_ipmitool(self.info, cmd)
        self.assertEqual(expected_output, actual_out)

    @mock.patch.object(subprocess, 'Popen')
    def test__exec_ipmitool_none(self, popen_mock):
        popen_mock.side_effect = Exception
        cmd = "fru print 0x2"
        actual_out = ipmi._exec_ipmitool(self.info, cmd)
        self.assertIsNone(actual_out)

    def test__parse_ipmi_nic_capacity(self):
        exec_output = constants.NIC_FRU_OUT
        expected_output = "1Gb"
        actual_out = ipmi._parse_ipmi_nic_capacity(exec_output)
        self.assertEqual(expected_output, actual_out)

    def test__parse_ipmi_nic_capacity_no_port_details(self):
        exec_output = constants.NIC_FRU_OUT_NO_PORT_DETAILS
        actual_out = ipmi._parse_ipmi_nic_capacity(exec_output)
        self.assertIsNone(actual_out)

    def test__parse_ipmi_nic_capacity_no_product_name(self):
        exec_output = constants.NIC_FRU_OUT_NO_PRODUCT_NAME
        actual_out = ipmi._parse_ipmi_nic_capacity(exec_output)
        self.assertIsNone(actual_out)



# Copyright 2017 Hewlett Packard Enterprise Development LP
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# Push power button action constants

PUSH_POWER_BUTTON_PRESS = 'press'
PUSH_POWER_BUTTON_PRESS_AND_HOLD = 'press and hold'

# BIOS Settings boot mode constants

BIOS_BOOT_MODE_LEGACY_BIOS = 'legacy bios'
BIOS_BOOT_MODE_UEFI = 'uefi'

# Persistent boot device for set

BOOT_SOURCE_TARGET_CD = 'Cd'
BOOT_SOURCE_TARGET_PXE = 'Pxe'
BOOT_SOURCE_TARGET_UEFI_TARGET = 'UefiTarget'
BOOT_SOURCE_TARGET_HDD = 'Hdd'

# BIOS Sriov constants

SRIOV_ENABLED = 'sriov enabled'
SRIOV_DISABLED = 'sriov disabled'

# Secure Boot current boot constants

SECUREBOOT_CURRENT_BOOT_ENABLED = 'enabled'
SECUREBOOT_CURRENT_BOOT_DISABLED = 'disabled'

# Secure Boot reset keys constants

SECUREBOOT_RESET_KEYS_DEFAULT = 'default'
SECUREBOOT_RESET_KEYS_DELETE_ALL = 'delete all'
SECUREBOOT_RESET_KEYS_DELETE_PK = 'delete pk'

TPM_PRESENT_ENABLED = 'Tpm present enabled'
TPM_PRESENT_DISABLED = 'Tpm present disabled'
TPM_NOT_PRESENT = 'Tpm not present'

# BIOS Cpu Virtualisation contants

CPUVT_ENABLED = 'cpu_vt enabled'
CPUVT_DISABLED = 'cpu_vt disabled'

# System supported boot mode contants

SUPPORTED_LEGACY_BIOS_ONLY = 'legacy bios only'
SUPPORTED_UEFI_ONLY = 'uefi only'
SUPPORTED_LEGACY_BIOS_AND_UEFI = 'legacy bios and uefi'

# Health related constants
HEALTH_STATE_ENABLED = 'enabled'
HEALTH_STATE_DISABLED = 'disabled'
HEALTH_OK = 'ok'
HEALTH_WARNING = 'warning'
HEALTH_CRITICAL = 'critical'

# Memory related constants
MEMORY_TYPE_NVDIMM_N = "nvdimm_n"
MEMORY_TYPE_DRAM = "dram"
MEMORY_DEVICE_TYPE_LOGICAL = "logical"
MEMORY_DEVICE_TYPE_DDR4 = "ddr4"

# System POST state related constants
POST_STATE_NULL = "null"
POST_STATE_UNKNOWN = "unknown"
POST_STATE_RESET = "reset"
POST_STATE_POWEROFF = "poweroff"
POST_STATE_INPOST = "inpost"
POST_STATE_INPOSTDISCOVERY = "inpostdiscoverycomplete"
POST_STATE_FINISHEDPOST = "finishedpost"



# Copyright 2017 Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

__author__ = 'HPE'

import collections

import six

from proliantutils import exception
from proliantutils.redfish.resources.system import constants as sys_cons


# Representation of supported boot modes
SupportedBootModes = collections.namedtuple(
    'SupportedBootModes', ['boot_mode_bios', 'boot_mode_uefi'])


def get_subresource_path_by(resource, subresource_path):
    """Helper function to find the resource path

    :param resource: ResourceBase instance from which the path is loaded.
    :param subresource_path: JSON field to fetch the value from.
            Either a string, or a list of strings in case of a nested field.
            It should also include the '@odata.id'
    :raises: MissingAttributeError, if required path is missing.
    :raises: ValueError, if path is empty.
    :raises: AttributeError, if json attr not found in resource
    """
    if isinstance(subresource_path, six.string_types):
        subresource_path = [subresource_path]
    elif not subresource_path:
        raise ValueError('"subresource_path" cannot be empty')

    body = resource.json
    for path_item in subresource_path:
        body = body.get(path_item, {})

    if not body:
        raise exception.MissingAttributeError(
            attribute='/'.join(subresource_path), resource=resource.path)

    if '@odata.id' not in body:
        raise exception.MissingAttributeError(
            attribute='/'.join(subresource_path) + '/@odata.id',
            resource=resource.path)

    return body['@odata.id']


def get_supported_boot_mode(supported_boot_mode):
    """Return bios and uefi support.

    :param supported_boot_mode: Supported boot modes
    :return: A tuple of 'true'/'false' based on bios and uefi
        support respectively.
    """
    boot_mode_bios = 'false'
    boot_mode_uefi = 'false'
    if (supported_boot_mode == sys_cons.SUPPORTED_LEGACY_BIOS_ONLY):
        boot_mode_bios = 'true'
    elif (supported_boot_mode == sys_cons.SUPPORTED_UEFI_ONLY):
        boot_mode_uefi = 'true'
    elif (supported_boot_mode == sys_cons.SUPPORTED_LEGACY_BIOS_AND_UEFI):
        boot_mode_bios = 'true'
        boot_mode_uefi = 'true'

    return SupportedBootModes(boot_mode_bios=boot_mode_bios,
                              boot_mode_uefi=boot_mode_uefi)


def get_allowed_operations(resource, subresource_path):
    """Helper function to get the HTTP allowed methods.

    :param resource: ResourceBase instance from which the path is loaded.
    :param subresource_path: JSON field to fetch the value from.
            Either a string, or a list of strings in case of a nested field.
    :returns: A list of allowed HTTP methods.
    """
    uri = get_subresource_path_by(resource, subresource_path)
    response = resource._conn.get(path=uri)
    return response.headers['Allow']


def is_operation_allowed(method, resource, subresouce_path):
    """Checks whether the operation is allowed for the resource.

    This method checks whether a HTTP method is allowed to be
    performed on the given sub resource path.
    :param method: A HTTP method. example: GET, PATCH, POST
    :param resource: ResourceBase instance from which the path is loaded.
    :param subresource_path: JSON field to fetch the value from.
            Either a string, or a list of strings in case of a nested field.
    :returns: True if the operation is allowed else False
    """
    return method in get_allowed_operations(resource, subresouce_path)


def max_safe(iterable):
    """Creates a wrapper over python max() function.

    This function is just a wrapper over pthon max().
    It catches the exceptions and let max() return without any error.
    """

    try:
        return max(iterable)
    except ValueError:
        # The TypeError is not caught here as that should be thrown.
        return 0
